/* SPDX-License-Identifier: GPL-2.0 */
/*
 * power_dsm.h
 *
 * dsm for power module
 *
 * Copyright (c) 2020-2020 Huawei Technologies Co., Ltd.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 */

#ifndef _POWER_DSM_H_
#define _POWER_DSM_H_

#include <dsm/dsm_pub.h>
#include <linux/list.h>

/* define dsm new id */
#define POWER_DSM_ERROR_BATT_ACR_OVER_THRESHOLD           0
#define POWER_DSM_ERROR_LOW_VOL_INT                       0
#define POWER_DSM_ERROR_NO_WATER_CHECK_BASE               0
#define POWER_DSM_ERROR_NO_TYPEC_4480_OVP                 0
#define POWER_DSM_ERROR_SWITCH_ATTACH                     0
#define POWER_DSM_ERROR_FCP_OUTPUT                        0
#define POWER_DSM_ERROR_FCP_DETECT                        0
#define POWER_DSM_ERROR_BATT_TEMP_LOW                     0
#define POWER_DSM_ERROR_BATT_SOC_CHANGE_FAST              0
#define POWER_DSM_ERROR_NO_TYPEC_CC_OVP                   0
#define POWER_DSM_ERROR_WIRELESS_TX_STATISTICS            0
#define POWER_DSM_ERROR_NO_SMPL                           0
#define POWER_DSM_ERROR_NON_STANDARD_CHARGER_PLUGGED      0
#define POWER_DSM_ERROR_NO_WATER_CHECK_IN_USB             0
#define POWER_DSM_ERROR_BATTERY_POLAR_ISHORT              0
#define POWER_DSM_ERROR_NO_CPU_BUCK_BASE                  0
#define POWER_DSM_ERROR_REFRESH_FCC_OUTSIDE               0
#define POWER_DSM_ERROR_BATT_TERMINATE_TOO_EARLY          0
#define POWER_DSM_ERROR_CHARGE_I2C_RW                     0
#define POWER_DSM_ERROR_WEAKSOURCE_STOP_CHARGE            0
#define POWER_DSM_ERROR_BOOST_OCP                         0
#define POWER_DSM_ERROR_NO_USB_SHORT_PROTECT_NTC          0
#define POWER_DSM_ERROR_NO_USB_SHORT_PROTECT              1
#define POWER_DSM_ERROR_NO_USB_SHORT_PROTECT_HIZ          2
#define POWER_DSM_ERROR_WIRELESS_BOOSTING_FAIL            0
#define POWER_DSM_ERROR_WIRELESS_CERTI_COMM_FAIL          0
#define POWER_DSM_ERROR_WIRELESS_CHECK_TX_ABILITY_FAIL    0
#define POWER_DSM_ERROR_WIRELESS_RX_OCP                   0
#define POWER_DSM_ERROR_WIRELESS_RX_OVP                   0
#define POWER_DSM_ERROR_WIRELESS_RX_OTP                   0
#define POWER_DSM_ERROR_WIRELESS_TX_POWER_SUPPLY_FAIL     0
#define POWER_DSM_ERROR_WIRELESS_TX_BATTERY_OVERHEAT      0
#define POWER_DSM_ERROR_WIRELESS_ERROR                    0
#define POWER_DSM_ERROR_ADAPTER_OVLT                      0
#define POWER_DSM_ERROR_ADAPTER_OCCURRENT                 0
#define POWER_DSM_ERROR_ADAPTER_OTEMP                     0
#define POWER_DSM_DIRECT_CHARGE_ADAPTER_OTP               0
#define POWER_DSM_DIRECT_CHARGE_VOL_ACCURACY              0
#define POWER_DSM_DIRECT_CHARGE_USB_PORT_LEAKAGE_CURRENT  0
#define POWER_DSM_DIRECT_CHARGE_FULL_PATH_RESISTANCE      0
#define POWER_DSM_DIRECT_CHARGE_VBUS_OVP                  0
#define POWER_DSM_DIRECT_CHARGE_REVERSE_OCP               0
#define POWER_DSM_DIRECT_CHARGE_OTP                       0
#define POWER_DSM_DIRECT_CHARGE_INPUT_OCP                 0
#define POWER_DSM_DIRECT_CHARGE_SC_FAULT_VBUS_OVP         0
#define POWER_DSM_DIRECT_CHARGE_SC_FAULT_TSBAT_OTP        0
#define POWER_DSM_DIRECT_CHARGE_SC_FAULT_TSBUS_OTP        0
#define POWER_DSM_DIRECT_CHARGE_SC_FAULT_TDIE_OTP         0
#define POWER_DSM_DIRECT_CHARGE_SC_FAULT_AC_OVP           0
#define POWER_DSM_DIRECT_CHARGE_SC_FAULT_VBAT_OVP         0
#define POWER_DSM_DIRECT_CHARGE_SC_FAULT_IBAT_OCP         0
#define POWER_DSM_DIRECT_CHARGE_SC_FAULT_IBUS_OCP         0
#define POWER_DSM_DIRECT_CHARGE_SC_FAULT_CONV_OCP         0
#define POWER_DSM_DIRECT_CHARGE_SC_FAULT_CFLY_SHORT       0
#define POWER_DSM_DIRECT_CHARGE_FULL_PATH_RESISTANCE_2ND  0
#define POWER_DSM_SOFT_RECHARGE_NO                        0
#define POWER_DSM_BATTERY_HEATING                         0
#define POWER_DSM_POWER_DEVICES_INFO                      0
#define POWER_DSM_BATTERY_DETECT_ERROR_NO                 0
#define POWER_DSM_OTG_VBUS_SHORT                          0
#define POWER_DSM_LIGHTSTRAP_STATUS                       0
#define POWER_DSM_PD_TYPEC_VBUS_VALID                     0
#define POWER_DSM_PD_TYPEC_VBUS_STILL_VALID               0
#define POWER_DSM_DA_BATTERY_ACR_ERROR_NO                 0
#define POWER_DSM_BATTERY_ROM_ID_CERTIFICATION_FAIL       0
#define POWER_DSM_BATTERY_IC_EEPROM_STATE_ERROR           0
#define POWER_DSM_BATTERY_IC_KEY_CERTIFICATION_FAIL       0
#define POWER_DSM_OLD_BOARD_AND_OLD_BATTERY_UNMATCH       0
#define POWER_DSM_OLD_BOARD_AND_NEW_BATTERY_UNMATCH       0
#define POWER_DSM_NEW_BOARD_AND_OLD_BATTERY_UNMATCH       0
#define POWER_DSM_BATTERY_NV_DATA_READ_FAIL               0
#define POWER_DSM_CERTIFICATION_SERVICE_IS_NOT_RESPONDING 0
#define POWER_DSM_UNMATCH_BATTERYS                        0
#define POWER_DSM_MULTI_CHARGE_CURRENT_RATIO_INFO         0
#define POWER_DSM_MULTI_CHARGE_CURRENT_RATIO_WARNING      0
#define POWER_DSM_MULTI_CHARGE_CURRENT_RATIO_ERROR        0
#define POWER_DSM_DIRECT_CHARGE_ERR_WITH_STD_CABLE        0
#define POWER_DSM_DIRECT_CHARGE_ERR_WITH_NONSTD_CABLE     0
#define POWER_DSM_DUAL_BATTERY_CURRENT_BIAS_DETECT        0
#define POWER_DSM_DUAL_BATTERY_OUT_OF_POSITION_DETECTION  0
#define POWER_DSM_DUAL_BATTERY_CAPACITY_DIFFERENT_DETECT  0
#define POWER_DSM_WLPS_GPIO_TXSW                          0
#define POWER_DSM_PMU_BMS_BATTERY_VOL_SOC_DISMATCH        0
#define POWER_DSM_CHARGE_SCHARGER_USB_PORT_VDROPMIN_WARNING               0
#define POWER_DSM_CHARGE_SCHARGER_SERIAL_PARALLEL_SWITCH_QSW2_ERROR       0
#define POWER_DSM_CHARGE_SCHARGER_SERIAL_PARALLEL_SWITCH_QSW1_ERROR       0
#define POWER_DSM_CHARGE_SCHARGER_SERIAL_CHARGE_QSW2_RCP_ERROR            0
#define POWER_DSM_CHARGE_SCHARGER_PARALLEL_ONE_BATTERY_DISCHARGE_WARNING  0
#define POWER_DSM_CHARGE_SCHARGER_SERIAL_CHARGE_ONE_BATTERY_ERR_WARNING   0
#define POWER_DSM_CHARGE_SCHARGER_LVC_PATH_WARNING                        0
#define POWER_DSM_CHARGE_SCHARGER_FUEL_GAUGE_PRECISION_WARNING            0
#define POWER_DSM_CHARGE_DYNAMIC_CV_TRIM_MAX                              0
#define POWER_DSM_CHARGE_DYNAMIC_CV_TRIM_INFO                             0
#define POWER_DSM_CHARGE_DYNAMIC_CV_CHARGER_DONE_VBAT_INFO                0

/* define dsm buffer size */
#define POWER_DSM_BUF_SIZE_0016                           16
#define POWER_DSM_BUF_SIZE_0032                           32
#define POWER_DSM_BUF_SIZE_0048                           48
#define POWER_DSM_BUF_SIZE_0128                           128
#define POWER_DSM_BUF_SIZE_0256                           256
#define POWER_DSM_BUF_SIZE_0512                           512
#define POWER_DSM_BUF_SIZE_1024                           1024
#define POWER_DSM_BUF_SIZE_2048                           2048
#define POWER_DSM_RESERVED_SIZE                           16


enum power_dsm_type {
	POWER_DSM_TYPE_BEGIN = 0,
	POWER_DSM_CPU_BUCK = POWER_DSM_TYPE_BEGIN,
	POWER_DSM_USB,
	POWER_DSM_BATTERY_DETECT,
	POWER_DSM_BATTERY,
	POWER_DSM_CHARGE_MONITOR,
	POWER_DSM_SUPERSWITCH,
	POWER_DSM_SMPL,
	POWER_DSM_PD_RICHTEK,
	POWER_DSM_PD,
	POWER_DSM_USCP,
	POWER_DSM_PMU_OCP,
	POWER_DSM_PMU_BMS,
	POWER_DSM_PMU_IRQ,
	POWER_DSM_VIBRATOR_IRQ,
	POWER_DSM_DIRECT_CHARGE_SC,
	POWER_DSM_FCP_CHARGE,
	POWER_DSM_MTK_SWITCH_CHARGE2,
	POWER_DSM_LIGHTSTRAP,
	POWER_DSM_TYPE_END,
};


struct power_dsm_dump {
	unsigned int type;
	int error_no;
	bool dump_enable;
	bool dump_always;
	const char *pre_buf;
	bool (*support)(void);
	void (*dump)(char *buf, unsigned int buf_len);
	bool (*check_error)(char *buf, unsigned int buf_len);
};

static inline struct dsm_client *power_dsm_get_dclient(unsigned int type)
{
	return NULL;
}

static inline int power_dsm_report_dmd(unsigned int type,
	int err_no, const char *buf)
{
	return 0;
}

static inline int power_dsm_report_fmd(unsigned int type,
	int err_no, const void *msg)
{
	return 0;
}

#define power_dsm_report_format_dmd(type, err_no, fmt, args...)

static inline void power_dsm_reset_dump_enable(
	struct power_dsm_dump *data, unsigned int size)
{
}

static inline void power_dsm_dump_data(
	struct power_dsm_dump *data, unsigned int size)
{
}

static inline void power_dsm_dump_data_with_error_no(
	struct power_dsm_dump *data, unsigned int size, int err_no)
{
}


static inline int power_dsm_report_hiview(unsigned int err_no, const char *key, const char *value)
{
	return 0;
}

#endif /* _POWER_DSM_H_ */
