

#include "wifi_ext_mark.h"
#include <linux/init.h>
#include <linux/types.h>
#include <linux/string.h>
#include <linux/uaccess.h>
#include <linux/netdevice.h>
#include <linux/netfilter_ipv4.h>
#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/errno.h>
#include <net/udp.h>
#include <linux/kernel.h> /* add for log */
#include <linux/ctype.h> /* add for tolower */
#include <linux/file.h>
#include <linux/rbtree.h>
#include <linux/inetdevice.h>
#include <linux/netdevice.h>
#include <linux/time.h>
#include <net/sock.h>
#include <hwnet/ipv4/wifipro_tcp_monitor.h>
#include <linux/spinlock.h> /* add for spinlock */
#include <linux/netlink.h> /* add for thread */
#include <uapi/linux/netlink.h> /* add for netlink */
#include <linux/kthread.h> /* add for thread */
#include <linux/version.h>

#define HID2D_VIDEO_MARK 0x5b

void mark_hid2d_skb(struct sk_buff *skb)
{
	struct udphdr *udph = NULL;
	struct tcphdr *tcph = NULL;
	struct iphdr *iph = ip_hdr(skb);

	if (g_hid2d_ip == 0 || g_hid2d_port == 0 || iph->daddr != g_hid2d_ip)
		return;
	if (iph->protocol == IPPROTO_UDP) {
		udph = udp_hdr(skb);
		if (udph->dest != g_hid2d_port && udph->source != g_hid2d_port)
			return;
	} else if (iph->protocol == IPPROTO_TCP) {
		tcph = tcp_hdr(skb);
		if (tcph->dest != g_hid2d_port && tcph->source != g_hid2d_port)
			return;
	} else {
		return;
	}

	skb->mark = HID2D_VIDEO_MARK;
}

static unsigned int wifiext_mark_hook_localout(void *ops, struct sk_buff *skb,
	const struct nf_hook_state *state)
{
	/* match the packet for optimization */
	if (skb != NULL)
		mark_hid2d_skb(skb);

	return NF_ACCEPT;
}

static struct nf_hook_ops net_hooks[] = {
	{
		.hook = wifiext_mark_hook_localout,
#if (KERNEL_VERSION(4, 4, 0) > LINUX_VERSION_CODE)
		.owner = THIS_MODULE,
#endif
		.pf = PF_INET,
		.hooknum = NF_INET_LOCAL_OUT,
		.priority = NF_IP_PRI_FILTER - 1,
	},
};

static int __init wifi_ext_mark_init(void)
{
	int ret = 0;

	/*
	 * Add a hook point on NF_INET_LOCAL_OUT,
	 * where we can get all the packets generated by local APP
	 */
#if (KERNEL_VERSION(4, 14, 0) > LINUX_VERSION_CODE)
	ret = nf_register_hooks(net_hooks, ARRAY_SIZE(net_hooks));
#else
	ret = nf_register_net_hooks(&init_net, net_hooks,
		ARRAY_SIZE(net_hooks));
#endif
	if (ret) {
		pr_info("nf_init ret=%d  ", ret);
		return -1;
	}

	return 0;
}

static void __exit wifi_ext_mark_exit(void)
{
#if (KERNEL_VERSION(4, 14, 0) > LINUX_VERSION_CODE)
	nf_unregister_hooks(net_hooks, ARRAY_SIZE(net_hooks));
#else
	nf_unregister_net_hooks(&init_net, net_hooks, ARRAY_SIZE(net_hooks));
#endif
}

module_init(wifi_ext_mark_init);
module_exit(wifi_ext_mark_exit);

MODULE_LICENSE("Dual BSD");
MODULE_AUTHOR("y00445093");
MODULE_DESCRIPTION("HW WIFI EXT MARK NF_HOOK");
MODULE_VERSION("1.0.1");
MODULE_ALIAS("HW WIFIX EXT 01");
