/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2020. All rights reserved.
 * Description: fbex common headers
 * Create : 2020/01/02
 */

#ifndef __FBE_CTRL_H_
#define __FBE_CTRL_H_

#include <linux/types.h>

#ifndef _unused
#define _unused __attribute__((unused))
#endif

enum FILE_ENCRY_CMD {
	SEC_FILE_ENCRY_CMD_ID_VOLD_ADD_IV = 0x1,
	SEC_FILE_ENCRY_CMD_ID_VOLD_DELETE_IV = 0x2,
	SEC_FILE_ENCRY_CMD_ID_LOCK_SCREEN = 0x3,
	SEC_FILE_ENCRY_CMD_ID_UNLOCK_SCREEN = 0x4,
	SEC_FILE_ENCRY_CMD_ID_KEY_RESTORE = 0x5,
	SEC_FILE_ENCRY_CMD_ID_NEW_SECE = 0x6,
	SEC_FILE_ENCRY_CMD_ID_GEN_METADATA = 0x7,
	SEC_FILE_ENCRY_CMD_ID_USER_LOGOUT = 0x8,
	SEC_FILE_ENCRY_CMD_ID_ENABLE_KDF = 0x9,
	SEC_FILE_ENCRY_CMD_ID_PRELOADING = 0xA,
	SEC_FILE_ENCRY_CMD_ID_MSPC_CHECK = 0xB,
	SEC_FILE_ENCRY_CMD_ID_STATUS_REPORT = 0xC,
#ifdef CONFIG_FBE3_1
	SEC_FILE_ENCRY_CMD_ID_GEN_SK = 0xD,
#endif
#ifdef CONFIG_PRODUCT_ARMPC
	SEC_FILE_ENCRY_CMD_ID_REWRAP = 0x10,
	SEC_FILE_ENCRY_CMD_ID_SET_MASTERKEY = 0x11,
	SEC_FILE_ENCRY_CMD_ID_DEL_MASTERKEY = 0x12,

	SEC_FILE_ENCRY_CMD_ID_MAX_SUPPORT = 0x20,
#else
	/* Reserve cmd id 0xC - 0xF for future using */
	SEC_FILE_ENCRY_CMD_ID_MAX_SUPPORT = 0x10,
#endif
};

#define SEC_FILE_ENCRY_CMD_ID_MASK (SEC_FILE_ENCRY_CMD_ID_MAX_SUPPORT - 1)
#define FILE_ECE       2

#define KEY_LEN      64 /* 64 * 8 = 512bits */
#define METADATA_LEN 16 /* 16 * 8 = 128bits */
#ifdef FILE_ENCRY_P384_USING
#define PUBKEY_LEN   0x61 /* pubkey len, 0x61 for p384 */
#else
#define PUBKEY_LEN   0x41 /* pubkey len, 0x41 for p256 */
#endif

struct fbex_sece_param {
	u32 cmd;
	u32 idx;
	u32 key_len;
	u32 iv_len;
	u8 *pubkey;
	u8 *metadata;
};

#ifdef CONFIG_FILE_BASED_ENCRYPTO
bool fbex_slot_clean(u32 slot);
void fbe3_flushcache_done(void);
u32 fbex_enable_kdf(void);
#ifdef CONFIG_FBE3_1
#define FILE_ENCRY_GEN_SK_TYPE 0x00000001
#define FILE_ENCRY_RESTORE_SK_TYPE 0x00000002
#define FBE_CA_MSP_AVAILABLE 0x5A5AA5A5
#define FBE_CA_MSP_UNAVAILABLE 0xA5A55A5A
u32 fbex_gen_sk(uint32_t type);
u32 fbe_ca_msp_available(u32 *flag);
#endif
u32 fbex_restore_key(void);
u32 get_metadata(u8 *buf, u32 len);
u32 get_metadata_sece(struct fbex_sece_param *param);
void fbex_set_status(u32 user, bool lock);
u32 fbex_ca_lock_screen(u32 user, u32 file);
u32 fbex_ca_unlock_screen(u32 user, u32 file, u8 *iv, u32 iv_len);
#else
static inline bool fbex_slot_clean(u32 slot) {return false; }
static inline void fbe3_flushcache_done(void) {}
static inline u32 fbex_enable_kdf(void) {return 0; }
static inline u32 fbex_gen_sk(uint32_t type _unused) {return 0; }
static u32 fbe_ca_msp_available(u32 *flag) {return 0; }
static inline u32 fbex_restore_key(void) {return 0; }
static inline u32 get_metadata(u8 *buf _unused, u32 len _unused) {return 0; }
static inline u32 get_metadata_sece(struct fbex_sece_param *param _unused)
{
	return 0;
}
static inline u32 fbex_ca_lock_screen(u32 user, u32 file) {return 0; }
static inline u32 fbex_ca_unlock_screen(u32 user, u32 file, u8 *iv, u32 iv_len)
{
	return 0;
}
static inline void fbex_set_status(u32 user, bool lock) {}
#endif


#endif /* __FBE_CTRL_H_ */
