/* SPDX-License-Identifier: GPL-2.0 */
/*
 * liblinux Vendor Hooks
 *
 * Copyright (c) 2022, Huawei Tech. Co., Ltd.
 */

#undef TRACE_SYSTEM
#define TRACE_SYSTEM liblinux

#define TRACE_INCLUDE_PATH trace/hooks
#if !defined(_TRACE_HOOK_LIBLINUX_H) || defined(TRACE_HEADER_MULTI_READ)
#define _TRACE_HOOK_LIBLINUX_H
/* Define struct should be ahead of vendor_hooks.h, where there may undef TRACE_HEADER_MULTI_READ */
#ifndef TRACE_HEADER_MULTI_READ
enum liblinux_dfx_mem_event {
	LIBLINUX_DFX_MEM_MIN,
	LIBLINUX_DFX_SLAB_ALLOC,
	LIBLINUX_DFX_SLAB_FREE,
	LIBLINUX_DFX_PAGE_ALLOC,
	LIBLINUX_DFX_PAGE_FREE,
	LIBLINUX_DFX_VMALLOC_ALLOC,
	LIBLINUX_DFX_VMALLOC_FREE,
	LIBLINUX_DFX_MEM_MAX,
};
#endif

#include <trace/hooks/vendor_hooks.h>

#define HOOK_ALWAYS_ON 1
#define HOOK_ALWAYS_OFF 0

#ifdef CONFIG_LIBLINUX
#define DECLARE_LDK_HOOK DECLARE_HOOK
#define DECLARE_LDK_RESTRICTED_HOOK DECLARE_RESTRICTED_HOOK
#else
#define DECLARE_LDK_HOOK(name, proto, args) \
	static inline void trace_##name(proto) {} \
	static inline bool trace_##name##_enabled(void) { return false; }

#define DECLARE_LDK_RESTRICTED_HOOK(name, proto, arg, cond) \
	DECLARE_LDK_HOOK(name, PARAMS(proto), PARAMS(args))
#endif

/*
 * kernel_init done hook
 */
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_kernel_init_done,
	     TP_PROTO(enum system_states system_state),
	     TP_ARGS(system_state), HOOK_ALWAYS_ON);

/*
 * ldk_rvh_dfx_call_mem used in:
 * mm/{slab.h, slub.c, slab_common.c}
 * mm/pal/page_alloc.c
 */
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_dfx_call_mem,
	     TP_PROTO(enum liblinux_dfx_mem_event ev_enum, void *addr,
		      unsigned long size, const char *slab_cache_name),
	     TP_ARGS(ev_enum, addr, size, slab_cache_name), HOOK_ALWAYS_ON);

#ifdef CONFIG_CMA
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_cma_add_section,
	TP_PROTO(unsigned long pfn, unsigned long count),
	TP_ARGS(pfn, count), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_cma_alloc,
	TP_PROTO(unsigned long *bitmap_no, const struct cma *cma, size_t count,
		 unsigned int align_order, int *ret),
	TP_ARGS(bitmap_no, cma, count, align_order, ret), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_cma_release,
	TP_PROTO(const struct cma *cma, const struct page *pages, unsigned int count),
	TP_ARGS(cma, pages, count), HOOK_ALWAYS_ON);
#endif /* CONFIG_CMA */

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_zone_pageset_init_late,
	TP_PROTO(int unuse),
	TP_ARGS(unuse), HOOK_ALWAYS_ON);

/* drivers/base/core.c */
#define LIBLINUX_VH_DEVICE_ADD	1U
#define LIBLINUX_VH_DEVICE_DEL	2U
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_device_notify,
	TP_PROTO(struct device *dev, unsigned int event),
	TP_ARGS(dev, event), HOOK_ALWAYS_ON);

/* kernel/task_work.c */
struct task_struct;
enum task_work_notify_mode;
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_task_work_check,
	     TP_PROTO(struct task_struct *task, enum task_work_notify_mode notify),
	     TP_ARGS(task, notify), HOOK_ALWAYS_ON);

/* kernel/workqueue.c */
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_set_vip_prio,
	TP_PROTO(struct task_struct *p, int prio),
	TP_ARGS(p, prio), HOOK_ALWAYS_ON);

struct worker_pool;
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_unbind_workers,
			TP_PROTO(struct worker_pool *pool, int cpu, struct mutex *lock, struct list_head *workers),
			TP_ARGS(pool, cpu, lock, workers), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_too_many_workers,
			    TP_PROTO(unsigned int nr_workers, bool *need_caching),
			    TP_ARGS(nr_workers, need_caching), HOOK_ALWAYS_ON);

/* fs/{char,block}_dev.c */
#define VH_REG_REGION	0
#define VH_UNREG_REGION	1
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_chrdev_region,
	     TP_PROTO(int action, unsigned int *major, unsigned int baseminor,
		      int minorct, const char *name, int *ret),
	     TP_ARGS(action, major, baseminor, minorct, name, ret), HOOK_ALWAYS_ON);
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_blkdev_region,
	     TP_PROTO(int action, unsigned int *major,
		      const char *name, int *ret),
	     TP_ARGS(action, major, name, ret), HOOK_ALWAYS_ON);

/* fs/configfs */
struct path;
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_gdentry,
	     TP_PROTO(const char *pathname, struct path *path, int *ret),
	     TP_ARGS(pathname, path, ret), HOOK_ALWAYS_ON);
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_pdentry,
	     TP_PROTO(void *sb, void *dentry),
	     TP_ARGS(sb, dentry), HOOK_ALWAYS_ON);

/* fs/procfs/generic.c */
#define VH_REG_PROCFS_PREPARE 0
#define VH_REG_PROCFS 1
struct proc_dir_entry;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_proc_register,
	     TP_PROTO(int action, struct proc_dir_entry *dp),
	     TP_ARGS(action, dp), HOOK_ALWAYS_ON);
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_remove_proc_entry,
	     TP_PROTO(struct proc_dir_entry *de),
	     TP_ARGS(de), HOOK_ALWAYS_ON);
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_remove_proc_subtree,
	     TP_PROTO(struct proc_dir_entry *de),
	     TP_ARGS(de), HOOK_ALWAYS_ON);

struct file_system_type;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_proc_root_init,
	     TP_PROTO(struct file_system_type *type),
	     TP_ARGS(type), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_devinet_sysctl_unregister,
	     TP_PROTO(unsigned int nsid, char *dev_name),
	     TP_ARGS(nsid, dev_name), HOOK_ALWAYS_ON);
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_addrconf_sysctl_unregister,
	     TP_PROTO(unsigned int nsid, char *dev_name),
	     TP_ARGS(nsid, dev_name), HOOK_ALWAYS_ON);

/* fs/proc/proc_sysctl.c */
struct ctl_table;
struct ctl_table_header;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_register_sysctl_table,
			TP_PROTO(const char *dir, struct ctl_table_header *head, struct ctl_table *table),
			TP_ARGS(dir, head, table), HOOK_ALWAYS_ON);

/* fs/kernfs */
struct kernfs_node;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_kernfs_add_one,
			TP_PROTO(struct kernfs_node *kn, void (*remove)(struct kernfs_node *), int *ret),
			TP_ARGS(kn, remove, ret), HOOK_ALWAYS_ON);
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_kernfs_put,
			TP_PROTO(struct kernfs_node *kn, int *ret),
			TP_ARGS(kn, ret), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_kernfs_notify,
			TP_PROTO(struct kernfs_node *kn),
			TP_ARGS(kn), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_kernfs_rename_ns,
			TP_PROTO(struct kernfs_node *kn, struct kernfs_node *new_parent,
				 const char *new_name, const void *new_ns, int *ret),
			TP_ARGS(kn, new_parent, new_name, new_ns, ret), HOOK_ALWAYS_ON);

/* fs/sysfs */
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_sysfs_init,
			TP_PROTO(struct kernfs_root *root, int *ret),
			TP_ARGS(root, ret), HOOK_ALWAYS_ON);

/* kernel/power */
struct pm_qos_constraints;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_pm_qos_notify,
			TP_PROTO(struct pm_qos_constraints *c, int value),
			TP_ARGS(c, value), HOOK_ALWAYS_ON);

/* net/network.c */
struct pernet_operations;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_proc_net_init,
			TP_PROTO(struct pernet_operations *ops),
			TP_ARGS(ops), HOOK_ALWAYS_ON);

#ifdef CONFIG_NET_NS
struct liblinux_procfs_create_ops;
struct liblinux_pal_kernfs_ops;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_liblinux_proc_create_net_data,
			TP_PROTO(struct proc_dir_entry *head_dp, struct liblinux_procfs_create_ops **create_ops,
			struct liblinux_pal_kernfs_ops **ops),
			TP_ARGS(head_dp, create_ops, ops), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_register_net_sysctl_table,
			TP_PROTO(struct ctl_table_header *head, struct liblinux_procfs_create_ops **create_ops,
			struct liblinux_pal_kernfs_ops **ops),
			TP_ARGS(head, create_ops, ops), HOOK_ALWAYS_ON);

struct net;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_set_net_nsid,
			TP_PROTO(struct net *net, struct net *old_net),
			TP_ARGS(net, old_net), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_net_cookie_init,
			TP_PROTO(struct net *net),
			TP_ARGS(net), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_proc_net_register,
			TP_PROTO(struct net *net, struct proc_dir_entry *netd, int *err),
			TP_ARGS(net, netd, err), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_proc_net_unregister,
			TP_PROTO(struct net *net),
			TP_ARGS(net), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_devhost_net_socket_get_ns_by_fd,
			TP_PROTO(int fd, struct net **net),
			TP_ARGS(fd, net), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_devhost_net_socket_get_ns_by_pid,
			TP_PROTO(int pid, struct net **net),
			TP_ARGS(pid, net), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_devhost_net_socket_get_netns,
			TP_PROTO(struct net **net),
			TP_ARGS(net), HOOK_ALWAYS_ON);

#endif

/* net/core net/ipv4 net/packet */
struct sock;
struct page_frag;
struct sk_buff;
struct flowi4;
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_sk_page_frag,
	     TP_PROTO(struct sock *sk, struct page_frag **pfrag),
	     TP_ARGS(sk, pfrag), HOOK_ALWAYS_ON);

struct xt_table_info;
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_ipt_do_table,
	     TP_PROTO(const struct xt_table_info *private,
			unsigned int hook, unsigned int *verdict),
	     TP_ARGS(private, hook, verdict), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_sk_vm_insert_page,
	     TP_PROTO(struct vm_area_struct *vma, unsigned long addr, struct page *page, int *ret),
	     TP_ARGS(vma, addr, page, ret), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_tcp_init_transfer,
	     TP_PROTO(struct sock *sk, int op, struct sk_buff *skb),
	     TP_ARGS(sk, op, skb), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_udp_send_skb,
	     TP_PROTO(struct sk_buff *skb, struct flowi4 *fl4, int *ret),
	     TP_ARGS(skb, fl4, ret), HOOK_ALWAYS_ON);

/* net/tls */
#include <linux/sockptr.h>
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_do_tls_setsockopt,
	     TP_PROTO(struct sock *sk, int optname, sockptr_t optval,
		      unsigned int optlen, int *ret),
	     TP_ARGS(sk, optname, optval, optlen, ret), HOOK_ALWAYS_ON);

/* drivers/base/power/wakeup.c */
struct wakeup_source;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_ws_debugfs_init,
			TP_PROTO(struct wakeup_source *ws),
			TP_ARGS(ws), HOOK_ALWAYS_ON);

/* drivers/base/power/wakeup.c */
typedef void (*split_counters_t)(unsigned int *cnt, unsigned int *inpr);
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_wakeup_source_activate,
			TP_PROTO(bool is_activate, split_counters_t split_counters_f),
			TP_ARGS(is_activate, split_counters_f),
			TP_CONDITION(HOOK_ALWAYS_ON)
);

/* platform_source/cee/drivers/devfreq/mem_lat/devfreq_devbw.c */
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_devfreq_add_devbw,
			TP_PROTO(const char **name),
			TP_ARGS(name), HOOK_ALWAYS_ON);

/* platform_source/cee/drivers/mips_mem.c */
struct platform_device;
struct mipsmem_cluster_policy;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_mips_mem_info,
			TP_PROTO(struct platform_device *pdev,
			struct mipsmem_cluster_policy *cluster_policy),
			TP_ARGS(pdev, cluster_policy),
			HOOK_ALWAYS_ON);

/* drivers/base/firmware_loader/main.c */
struct device;
struct fw_priv;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_fw_get_filesystem_firmware,
			TP_PROTO(struct device *device, struct fw_priv *fw_priv,
			const char *suffix,
			int (*decompress)(struct device *dev,
					  struct fw_priv *fw_priv,
					  size_t in_size,
					  const void *in_buffer),
			const char * const *fw_path,
			size_t fw_path_size, int *ret),
			TP_ARGS(device, fw_priv, suffix, decompress, fw_path, fw_path_size, ret),
			HOOK_ALWAYS_ON);

/* arch/liblinux/mm/page_alloc.c */
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_free_pages,
			TP_PROTO(void *page, unsigned int order, int *ret),
			TP_ARGS(page, order, ret),
			TP_CONDITION(HOOK_ALWAYS_ON));

DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_hisp_alloc_largest_pages,
			TP_PROTO(unsigned int order, gfp_t *gfp_mask),
			TP_ARGS(order, gfp_mask),
			TP_CONDITION(HOOK_ALWAYS_ON));

/* drivers/usb/gadget/function/f_fs.c */
struct sg_table;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_ffs_build_sg_list,
			TP_PROTO(struct sg_table *sgt, size_t sz, void **vaddr),
			TP_ARGS(sgt, sz, vaddr),
			TP_CONDITION(HOOK_ALWAYS_ON));
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_ffs_free_sg_list,
			TP_PROTO(struct sg_table *sgt, void* vaddr),
			TP_ARGS(sgt, vaddr),
			TP_CONDITION(HOOK_ALWAYS_ON));

/* platform_source/basicplatform/drivers/rpmb/rpmb_ufs.c &
 * platform_source/basicplatform/block/blk_scsi_kern.c */
struct bsg_class_device;
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_get_rpmb_bcd,
                        TP_PROTO(const char *devname, struct bsg_class_device *bcd),
                        TP_ARGS(devname, bcd), HOOK_ALWAYS_ON);

struct ufs_blk_ioc_rpmb_data;
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_ufs_bsg_ioctl_rpmb,
			TP_PROTO(struct ufs_blk_ioc_rpmb_data *rdata, bool is_write,
				 int id, int source, int *ret),
			TP_ARGS(rdata, is_write, id, source, ret), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_sel_load_policy,
			TP_PROTO(const char *buffer, unsigned long size, ssize_t *ret),
			TP_ARGS(buffer, size, ret), HOOK_ALWAYS_ON);

#ifdef CONFIG_SECURITY_SELINUX_DEVELOP
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_sel_update_enforce,
			TP_PROTO(int value, ssize_t *ret),
			TP_ARGS(value, ret), HOOK_ALWAYS_ON);

#endif /* CONFIG_SECURITY_SELINUX_DEVELOP */
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_sel_update_cache_threshold,
			TP_PROTO(int value, ssize_t *ret),
			TP_ARGS(value, ret), HOOK_ALWAYS_ON);

struct timekeeper;
struct timespec64;
/* timekeeping_update function pointer */
typedef void (*tk_update_func_t)(struct timekeeper *tk, unsigned int actions);
/* tk_set_wall_to_mono function pointer */
typedef void (*tk_set_wtm_func_t)(struct timekeeper *tk, struct timespec64 wtm);
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_tk_init,
			TP_PROTO(struct timekeeper *tk, void *seq,
				 void *tk_lock, tk_update_func_t tk_update,
				 tk_set_wtm_func_t tk_set_wtm),
			TP_ARGS(tk, seq, tk_lock, tk_update, tk_set_wtm), HOOK_ALWAYS_ON);

/* drivers/dma-buf/heaps/page_pool.c */
struct dmabuf_page_pool;
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_dmabuf_page_pool_do_shrink,
			TP_PROTO(struct dmabuf_page_pool *pool, bool high, int nr_to_scan, int *freed_out),
			TP_ARGS(pool, high, nr_to_scan, freed_out), HOOK_ALWAYS_ON);
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_dmabuf_page_pool_free_pages,
			TP_PROTO(struct dmabuf_page_pool *pool, struct page *page),
			TP_ARGS(pool, page), HOOK_ALWAYS_ON);

/* drivers/connectivity/hi11xx/hi1105/wifi/hmac/hmac_auto_adjust_freq.c */
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_hmac_calc_throughput_aspm,
			TP_PROTO(unsigned int rx_throughput_mbps),
			TP_ARGS(rx_throughput_mbps), HOOK_ALWAYS_ON);

/* arch/liblinux/kernel/kthread.c */
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_init_task,
			TP_PROTO(struct task_struct *tsk),
			TP_ARGS(tsk), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_exit_task,
			TP_PROTO(struct task_struct *tsk),
			TP_ARGS(tsk), HOOK_ALWAYS_ON);

/* arch/liblinux/kernel/fs/namei.c */
struct filename;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_user_path_create,
			TP_PROTO(const char __user *pathname, struct path *path,
				 unsigned int lookup_flags, struct dentry **dentry),
			TP_ARGS(pathname, path, lookup_flags, dentry), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_user_path_at,
			TP_PROTO(const char __user *pathname, struct path *path, int *ret),
			TP_ARGS(pathname, path, ret), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_touch_atime,
			TP_PROTO(struct path *path),
			TP_ARGS(path), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_done_path_create,
			TP_PROTO(struct path *path, struct dentry *dentry),
			TP_ARGS(path, dentry), HOOK_ALWAYS_ON);

/* arch/liblinux/kernel/bpf.c */
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_bpf_cmd_check,
			TP_PROTO(int cmd, int *ret),
			TP_ARGS(cmd, ret), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_cgroup_bpf_enabled_key_dec,
			TP_PROTO(unsigned int type),
			TP_ARGS(type), HOOK_ALWAYS_ON);

struct bpf_prog;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_cgroup_bpf_enabled_key_inc,
			TP_PROTO(struct bpf_prog *old_prog, unsigned int type),
			TP_ARGS(old_prog, type), HOOK_ALWAYS_ON);

/* arch/liblinux/lib/uaccess.c */
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_copy_to_user,
			TP_PROTO(void __user *to, const void *from,
				unsigned long n, int *ret),
			TP_ARGS(to, from, n, ret), HOOK_ALWAYS_ON);

/* samples/ldk/cred.c */
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_syscall_exit,
			TP_PROTO(struct task_struct *tsk),
			TP_ARGS(tsk), HOOK_ALWAYS_ON);

/* drivers/tty/pty.c */
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_devpts_get_priv,
			    TP_PROTO(int index, void **ret),
			    TP_ARGS(index, ret), HOOK_ALWAYS_ON);

/* net/xdp/xdp_umem.c */
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_xdp_pin_user_pages,
	     TP_PROTO(unsigned long start, unsigned long nr_pages, unsigned int gup_flags,
                        struct page **pages, struct vm_area_struct **vmas, long *ret),
	     TP_ARGS(start, nr_pages, gup_flags, pages, vmas, ret), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_xdp_unpin_user_pages,
             TP_PROTO(struct page **pages, unsigned long npages, bool make_dirty),
             TP_ARGS(pages, npages, make_dirty), HOOK_ALWAYS_ON);

/* arch/arm64/kernel/insn.c */
DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_patch_map,
			    TP_PROTO(void *addr, int *ret),
			    TP_ARGS(addr, ret), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_patch_unmap,
			    TP_PROTO(void *addr, int *ret),
			    TP_ARGS(addr, ret), HOOK_ALWAYS_ON);

DECLARE_LDK_RESTRICTED_HOOK(ldk_rvh_initcall_blacklist,
			    TP_PROTO(initcall_t fn, int *ret),
			    TP_ARGS(fn, ret), HOOK_ALWAYS_ON);

/* platform_source/basicplatform/block/mas_blk_hm.c */
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_sd_proxy_block_event_cmd,
			    TP_PROTO(int event_id),
			    TP_ARGS(event_id), HOOK_ALWAYS_ON);

/* drivers/thermal/thermal_core.c */
struct device;
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_thermal_zone_device_set_mode,
			    TP_PROTO(struct device *dev, int mode),
			    TP_ARGS(dev, mode), HOOK_ALWAYS_ON);

/* arch/liblinux/mm/cache.c */
DECLARE_LDK_RESTRICTED_HOOK(ldk_vh_dev_set_user_inval_cache,
			TP_PROTO(struct device *dev),
			TP_ARGS(dev), HOOK_ALWAYS_ON);

/* NOTE: ALL LDK hooks must be places above this line */
#endif /* !defined(_TRACE_HOOK_LIBLINUX_H) || defined(TRACE_HEADER_MULTI_READ) */

/* This part must be outside protection */
#include <trace/define_trace.h>
